import * as react_jsx_runtime from 'react/jsx-runtime';
import { ComponentType, CSSProperties, ReactElement } from 'react';

interface Attribute {
    name: string;
    scope: string;
}
interface RendererProp<T> {
    column: ColumnHeader<T>;
    item?: T;
    [key: string]: any;
}
interface ClassesOverrides {
    classes: Record<string, string>;
}
interface ColumnHeader<T> {
    classes?: ClassesOverrides;
    component: ComponentType<RendererProp<T> & ClassesOverrides>;
    title: string;
    attribute: Attribute;
    sortable: boolean;
    customize?: () => void;
    style?: CSSProperties;
    textRender?: (props: RendererProp<T>) => string | ReactElement;
}
interface SortOptions {
    direction?: string;
    key?: string;
}
interface ListState {
    page?: number;
    perPage?: number;
    selection?: number[];
    sort?: SortOptions;
    total: number;
}
interface IdAttribute {
    attribute: string;
    scope: string;
}
type wID = {
    id: string;
};
interface CommonListProps<T extends wID> {
    listItems: T[];
    columnHeaders: ColumnHeader<T>[];
    customColumnSizes?: Attribute[];
    onExpandClick: (item: T) => void;
    onResizeColumns: ((columns: {
        size: number;
        attribute: Attribute;
    }) => void) | false;
    onPageChange: (event: MouseEvent | null, page: number) => void;
    onSelect: ((rows: number[]) => void) | false;
    onSort?: (attr: Attribute | object) => void;
    onChangeRowsPerPage: (perPage: number) => void;
    pageLoading: boolean;
    PaginationProps?: object;
    idAttribute?: IdAttribute;
    listState: ListState;
    sortingNotes?: {
        [key: string]: string;
    };
    ListItemComponent: ComponentType<ListItemComponentProps<T>>;
}
interface ListItemComponentProps<T> {
    columnHeaders: ColumnHeader<T>[];
    listItem: T;
    listState: ListState;
    idAttribute?: IdAttribute;
    index: number;
    key: string;
    onClick: (item: T) => void;
    onRowSelect: (selectedRow: T) => void;
    selectable: boolean;
    selected: boolean;
}
declare const minCellWidth = 150;
declare const calculateResizeChange: ({ columnElements, columnHeaders, e, index, prev, selectable }: {
    columnElements: any;
    columnHeaders: any;
    e: any;
    index: any;
    prev: any;
    selectable: any;
}) => any;
declare const CommonList: <T extends wID>(props: CommonListProps<T>) => react_jsx_runtime.JSX.Element;

export { type ClassesOverrides, type ColumnHeader, CommonList, type ListItemComponentProps, type RendererProp, calculateResizeChange, minCellWidth };
