"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/sockethook.js
var sockethook_exports = {};
__export(sockethook_exports, {
  blobToString: () => blobToString,
  byteArrayToString: () => byteArrayToString,
  useSession: () => useSession
});
module.exports = __toCommonJS(sockethook_exports);
var import_react = require("react");
var import_constants = require("@northern.tech/store/constants");
var import_msgpack5 = __toESM(require("msgpack5"), 1);
var import_universal_cookie = __toESM(require("universal-cookie"), 1);
var cookies = new import_universal_cookie.default();
var MessagePack = (0, import_msgpack5.default)();
var byteArrayToString = (body) => String.fromCharCode(...body);
var blobToString = (blob) => new Promise((resolve) => {
  let fr = new FileReader();
  fr.onload = () => {
    resolve(fr.result);
  };
  fr.readAsArrayBuffer(blob);
});
var useSession = ({ onClose, onHealthCheckFailed, onMessageReceived, onNotify, onOpen, token }) => {
  const [sessionId, setSessionId] = (0, import_react.useState)();
  const healthcheckTimeout = (0, import_react.useRef)();
  const socketRef = (0, import_react.useRef)();
  const sendMessage = (0, import_react.useCallback)(({ typ, body, props }) => {
    if (!socketRef.current) {
      return;
    }
    const proto_header = { proto: import_constants.DEVICE_MESSAGE_PROTOCOLS.Shell, typ, sid: socketRef.current.sessionId, props };
    const encodedData = MessagePack.encode({ hdr: proto_header, body });
    socketRef.current.send(encodedData);
  }, []);
  const close = (0, import_react.useCallback)(() => {
    if (!socketRef.current || socketRef.current?.readyState !== WebSocket.OPEN) {
      return;
    }
    sendMessage({ typ: import_constants.DEVICE_MESSAGE_TYPES.Stop, body: {}, props: {} });
    socketRef.current.close();
    setSessionId();
  }, [sendMessage]);
  const healthcheckFailed = (0, import_react.useCallback)(() => {
    onHealthCheckFailed();
    close();
  }, [close, onHealthCheckFailed]);
  const onSocketMessage = (0, import_react.useCallback)(
    (event) => blobToString(event.data).then((data) => {
      const {
        hdr: { props = {}, proto, sid, typ },
        body
      } = MessagePack.decode(data);
      if (proto !== import_constants.DEVICE_MESSAGE_PROTOCOLS.Shell) {
        return;
      }
      switch (typ) {
        case import_constants.DEVICE_MESSAGE_TYPES.New: {
          if (props.status == WebSocket.CLOSING) {
            onNotify(`Error: ${byteArrayToString(body)}`);
            setSessionId();
            return close();
          } else {
            clearTimeout(healthcheckTimeout.current);
            healthcheckTimeout.current = setTimeout(healthcheckFailed, 65 * import_constants.TIMEOUTS.oneSecond);
            socketRef.current.sessionId = sid;
            return setSessionId(sid);
          }
        }
        case import_constants.DEVICE_MESSAGE_TYPES.Shell:
          return onMessageReceived(body);
        case import_constants.DEVICE_MESSAGE_TYPES.Stop:
          return close();
        case import_constants.DEVICE_MESSAGE_TYPES.Ping: {
          if (healthcheckTimeout.current) {
            clearTimeout(healthcheckTimeout.current);
          }
          sendMessage({ typ: import_constants.DEVICE_MESSAGE_TYPES.Pong });
          const timeout = parseInt(props.timeout);
          if (timeout > 0) {
            healthcheckTimeout.current = setTimeout(healthcheckFailed, timeout * import_constants.TIMEOUTS.oneSecond);
          }
          return;
        }
        default:
          break;
      }
    }),
    [close, healthcheckFailed, onMessageReceived, onNotify, sendMessage]
  );
  const onSocketError = (0, import_react.useCallback)(
    (error) => {
      onNotify(`WebSocket error: ${error.message}`);
      close();
      clearTimeout(healthcheckTimeout.current);
    },
    [close, onNotify]
  );
  const onSocketOpen = (0, import_react.useCallback)(() => {
    sendMessage({ typ: import_constants.DEVICE_MESSAGE_TYPES.New, props: {} });
    onOpen(true);
  }, [onOpen, sendMessage]);
  const onSocketClose = (0, import_react.useCallback)(
    (e) => {
      console.log("closing");
      onClose(e);
      close();
    },
    [close, onClose]
  );
  const connect = (0, import_react.useCallback)(
    (deviceId) => {
      const uri = `wss://${window.location.host}${import_constants.apiUrl.v1}/deviceconnect/devices/${deviceId}/connect`;
      setSessionId();
      cookies.set("JWT", token, { path: "/", secure: true, sameSite: "strict", maxAge: 5 });
      try {
        socketRef.current = new WebSocket(uri);
      } catch (error) {
        console.log(error);
      }
    },
    [token]
  );
  (0, import_react.useEffect)(() => {
    if (!socketRef.current) {
      return;
    }
    socketRef.current.addEventListener("close", onSocketClose);
    socketRef.current.addEventListener("error", onSocketError);
    socketRef.current.addEventListener("message", onSocketMessage);
    socketRef.current.addEventListener("open", onSocketOpen);
    return () => {
      socketRef.current.removeEventListener("close", onSocketClose);
      socketRef.current.removeEventListener("error", onSocketError);
      socketRef.current.removeEventListener("message", onSocketMessage);
      socketRef.current.removeEventListener("open", onSocketOpen);
    };
  }, [onSocketClose, onSocketError, onSocketMessage, onSocketOpen, socketRef.current?.readyState]);
  return [connect, sendMessage, close, socketRef.current?.readyState ?? WebSocket.CLOSED, sessionId];
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  blobToString,
  byteArrayToString,
  useSession
});
//# sourceMappingURL=sockethook.cjs.map